@@ -1,5 +1,6 @@
-/* FIGARO'S PASSWORD MANAGER (FPM)
+/* FIGARO'S PASSWORD MANAGER 2 (FPM2)
  * Copyright (C) 2000 John Conneely
+ * Copyright (C) 2008 Aleš Koval
  * 
  * FPM is open source / free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,17 +31,20 @@
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <string.h> 
+#include <stdlib.h>
+#include <unistd.h>
 
 /* GLOBAL VARIABLES */
 GtkWidget* glb_win_app;   /* The main window for the application   */
-GtkCList* glb_clist_main; /* Password CList on main window         */
+GtkTreeView* glb_clist_main; /* Password CList on main window         */
 
 GtkWidget* glb_win_misc;  /* Misc window, used for dialogs         */
 
 GtkWidget* glb_win_edit;  /* The edit window for a password item   */
 GtkWidget* glb_win_import; /* The import window                    */
 fpm_data* glb_edit_data;  /* The password item being edited now    */
-gint glb_num_row ;        /* The total number of rows in the CList */
+gint glb_num_row;         /* The total number of rows in the CList */
 gint glb_cur_row;         /* The last item clicked on in the CList */
 gchar* glb_filename;      /* The location of the pasword file.     */
 gboolean glb_dirty;       /* Have changes been made?               */
@@ -48,6 +52,9 @@ GTimer* glb_timer_click;  /* Timer used to check for double clicks */
 gint glb_click_count;	  /* Click count used for double clicks    */
 gchar* new_salt;	  /* Salt to use on next save.             */
 gchar* old_salt;	  /* Salt to used on last save.            */
+gboolean glb_click_btn;	  /* Right button clicked -> context menu  */
+
+fpm_ini* glb_fpm_ini;	  /* Store FPM settings. */
 
 static gchar* glb_filename_lock; /* The location of the pasword loc kfile. */
 
@@ -113,10 +120,10 @@ lock_fpm_file(gchar* glb_filename)
 	if ( stat(pwd_dir_name, &fs_stat) )
 	{
 		cmd = g_strdup_printf("mkdir %s", pwd_dir_name);
-		gnome_execute_shell(NULL, cmd);
+		fpm_execute_shell(cmd);
 		g_free(cmd);
 		cmd = g_strdup_printf("chmod 0700 %s", pwd_dir_name);
-		gnome_execute_shell(NULL, cmd);
+		fpm_execute_shell(cmd);
 		g_free(cmd);
 		/* Wait for the shell commands to finish */
 		sleep(1);
@@ -126,10 +133,9 @@ lock_fpm_file(gchar* glb_filename)
 
 	if ( ( lock_fd = open( glb_filename_lock, O_CREAT | O_RDONLY, 0600 )) < 0 )
 	{
-		dia_str = g_strdup_printf("Could not open %s", 
+		dia_str = g_strdup_printf(_("Could not open %s"), 
 			glb_filename_lock);
-		glb_win_misc = gnome_error_dialog(dia_str);
-		gnome_dialog_run(GNOME_DIALOG(glb_win_misc));
+		fpm_message(GTK_WINDOW(glb_win_app), dia_str, GTK_MESSAGE_ERROR);
 
 #ifdef FPM_LINK_LOCK
 		unlink(glb_filename_lock_pid);
@@ -153,10 +159,9 @@ lock_fpm_file(gchar* glb_filename)
 	if ( flock(lock_fd, LOCK_EX | LOCK_NB) )
 #endif
 	{
-		dia_str = g_strdup_printf("Could not lock %s\nFile already locked.", 
+		dia_str = g_strdup_printf(_("Could not lock %s\nFile already locked."), 
 			glb_filename_lock);
-		glb_win_misc = gnome_error_dialog(dia_str);
-		gnome_dialog_run(GNOME_DIALOG(glb_win_misc));
+		fpm_message(GTK_WINDOW(glb_win_app), dia_str, GTK_MESSAGE_ERROR);
 		printf("%s\n", dia_str);
 		exit(1);
 	}
@@ -171,20 +176,18 @@ lock_fpm_file(gchar* glb_filename)
 #ifdef FPM_LINK_LOCK
 	if ( link(glb_filename_lock, glb_filename_lock_pid) )
 	{
-		dia_str = g_strdup_printf("Could not create a link to %s", 
+		dia_str = g_strdup_printf(_("Could not create a link to %s"), 
 			glb_filename_lock);
-		glb_win_misc = gnome_error_dialog(dia_str);
-		gnome_dialog_run(GNOME_DIALOG(glb_win_misc));
+		fpm_message(GTK_WINDOW(glb_win_app), dia_str, GTK_MESSAGE_ERROR);
 		unlink(glb_filename_lock_pid);
 		exit(1);
 	}
 
 	if ( fstat(lock_fd, &fs_stat) )
 	{
-		dia_str = g_strdup_printf("Could not fstat %s", 
+		dia_str = g_strdup_printf(_("Could not fstat %s"), 
 			glb_filename_lock);
-		glb_win_misc = gnome_error_dialog(dia_str);
-		gnome_dialog_run(GNOME_DIALOG(glb_win_misc));
+		fpm_message(GTK_WINDOW(glb_win_app), dia_str, GTK_MESSAGE_ERROR);
 		unlink(glb_filename_lock_pid);
 		printf("Lock already exists\n");
 		exit(1);
@@ -192,11 +195,10 @@ lock_fpm_file(gchar* glb_filename)
 
 	if ( fs_stat.st_nlink > 2 )
 	{
-		dia_str = g_strdup_printf("Lock file %s exists.\nIf you know "
+		dia_str = g_strdup_printf(_("Lock file %s exists.\nIf you know "
 			"this to be an error, clean up the\nlock files in"
-			"the directory %s.", glb_filename_lock, pwd_dir_name);
-		glb_win_misc = gnome_error_dialog(dia_str);
-		gnome_dialog_run(GNOME_DIALOG(glb_win_misc));
+			"the directory %s."), glb_filename_lock, pwd_dir_name);
+		fpm_message(GTK_WINDOW(glb_win_app), dia_str, GTK_MESSAGE_ERROR);
 		unlink(glb_filename_lock_pid);
 		exit(1);
 	}
@@ -213,6 +215,8 @@ unlock_fpm_file(void)
 	}
 #ifdef FPM_LINK_LOCK
 	unlink(glb_filename_lock_pid);
+#else
+	unlink(glb_filename_lock);
 #endif /* FPM_LINK_LOCK */
 }
 
@@ -249,7 +253,8 @@ fpm_edit_passitem(GtkWidget** win_edit_ptr, fpm_data* data)
 
   GtkCombo* combo;
   gchar cleartext[FPM_PASSWORD_LEN+1] = {0};
-  GtkText* notes;
+  GtkTextView* notes;
+  GtkTextBuffer*  buffer;
   int ix;
 
   /* Create new dialog box */
@@ -283,8 +288,10 @@ fpm_edit_passitem(GtkWidget** win_edit_ptr, fpm_data* data)
   fpm_set_entry(*win_edit_ptr, "entry_password", cleartext);
   memset(cleartext, 0, FPM_PASSWORD_LEN);
 
-  notes = GTK_TEXT(lookup_widget(*win_edit_ptr, "text_notes"));
-  gtk_text_insert(notes, NULL, NULL, NULL, data->notes, -1);
+  notes = GTK_TEXT_VIEW(lookup_widget(*win_edit_ptr, "text_notes"));
+
+  buffer = gtk_text_view_get_buffer(notes);
+  gtk_text_buffer_insert_at_cursor(buffer, data->notes ,strlen(data->notes));
   
   gtk_widget_show(*win_edit_ptr);
   gtk_widget_set_sensitive(glb_win_app, FALSE);
@@ -295,15 +302,21 @@ fpm_edit_passitem(GtkWidget** win_edit_ptr, fpm_data* data)
 void
 fpm_save_passitem(GtkWidget* win_edit, fpm_data* data)
 {
-  gint row;
-  char* blank_row[FPM_NUM_COL];
-  gint i;
+  gint row = -1;
   gchar* orig_plaintext;
   gchar plaintext[FPM_PASSWORD_LEN+1] = {0};
   GtkEntry* entry;
-  GtkEditable* notes;
+  GtkTextView* notes;
   gchar *dia_str;
 
+  fpm_data* data1;
+
+  GtkTextIter start, end;
+  GtkTextBuffer *buffer;
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  gboolean valid;
+  
   /* Set dirty flag so we know we need to save later. */
   glb_dirty = TRUE;
 
@@ -319,17 +332,17 @@ fpm_save_passitem(GtkWidget* win_edit, fpm_data* data)
 
   /* Update password */
   entry = GTK_ENTRY(lookup_widget(win_edit, "entry_password"));
-  orig_plaintext = gtk_entry_get_text(entry);
+
+  orig_plaintext = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
+
   if ( strlen(orig_plaintext) > (FPM_PASSWORD_LEN -1 ))
   {
-	GtkWidget *dw;
   	dia_str = g_malloc0(256);
-	sprintf(dia_str, "Password exceeded limit of %d characters"
-		"\nYour password of %d characters was truncated.", 
-		FPM_PASSWORD_LEN-1, strlen(orig_plaintext));
-	dw = gnome_error_dialog(dia_str);
-	gnome_dialog_run(GNOME_DIALOG(dw));
-	
+	sprintf(dia_str, _("Password exceeded limit of %d characters"
+		"\nYour password of %d characters was truncated."), 
+		FPM_PASSWORD_LEN-1, (int) strlen(orig_plaintext));
+
+	fpm_message(GTK_WINDOW(win_edit), dia_str, GTK_MESSAGE_INFO);
 	g_free(dia_str);
 
   }
@@ -339,55 +352,52 @@ fpm_save_passitem(GtkWidget* win_edit, fpm_data* data)
   memset(orig_plaintext, 0, strlen(orig_plaintext));
   memset(plaintext, 0, FPM_PASSWORD_LEN);
 
+  g_free(orig_plaintext);
+
   /* Update notes */
-  notes = GTK_EDITABLE(lookup_widget(win_edit, "text_notes"));
+  notes = GTK_TEXT_VIEW(lookup_widget(win_edit, "text_notes"));
+  gtk_text_view_set_editable (GTK_TEXT_VIEW (notes), TRUE);
   g_free(data->notes);
-  data->notes=gtk_editable_get_chars(notes, 0, -1);
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (notes));
+  gtk_text_buffer_get_bounds (buffer, &start, &end);
+  data->notes = gtk_text_iter_get_text (&start, &end);
+     
+  model = gtk_tree_view_get_model(glb_clist_main);
+  valid = gtk_tree_model_get_iter_first(model, &iter);
 
-  row = gtk_clist_find_row_from_data(glb_clist_main, data);
+  while (valid) {
+    gtk_tree_model_get(model, &iter, FPM_COL_DATA_POINTER, &data1, -1);
+    if( data == data1 ) {
+	row = atoi(gtk_tree_model_get_string_from_iter(model, &iter));
+    }
+    valid = gtk_tree_model_iter_next (model, &iter);
+  }
 
-  /* Check to see if this is a new entry */ 
   if (row<0)
   {
-
-    /* Construct blank row */
-    for(i=0;i<FPM_NUM_COL; i++) blank_row[i]=NULL;
-
-    /* Append row to end of CList */
-    gtk_clist_append(glb_clist_main, blank_row);
-
     /* Keep track of the number of rows in the CList */
     glb_num_row++;
 
     /* Update the current row */
     row = glb_num_row-1;
 
-    /* Associate this row with the data structure. */
-    gtk_clist_set_row_data(glb_clist_main, row, data);
-
     /* Also, add this row to our main GList */
     glb_pass_list=g_list_append(glb_pass_list, data);
 
     g_print("inserting...\n");
 
+    if (glb_fpm_ini->save_on_add)
+	passfile_save(glb_filename);
+  }
+  else
+  { 
+    g_print("updating...\n");
+    if (glb_fpm_ini->save_on_change)
+	passfile_save(glb_filename);
   }
-  else g_print("updating...\n");
-
-  /* Update Visible CList Fields */
-  gtk_clist_set_text(glb_clist_main, row, FPM_COL_TITLE, data->title);
-  gtk_clist_set_text(glb_clist_main, row, FPM_COL_URL, data->arg);
-  gtk_clist_set_text(glb_clist_main, row, FPM_COL_USER, data->user);
 
   /* Update Category drop down list on main screen */
   fpm_clist_populate_cat_list();
-
-  /* Question to save entry after modifying. Remove passfile_save below
-     if you uncomment this.
-  gnome_question_dialog_modal("Do you want to save changes?",
-    fpm_dialog_answer_save, NULL);
-  */
-
-  passfile_save(glb_filename);
 }
 
 
@@ -398,7 +408,7 @@ fpm_get_entry(GtkWidget* win, gchar* entry_name)
   GtkEntry* entry_field;
 
   entry_field = GTK_ENTRY(lookup_widget (win, entry_name));
-  tmp = gtk_entry_get_text(entry_field);
+  tmp = gtk_editable_get_chars (GTK_EDITABLE(entry_field), 0, -1);
   return (g_strdup(tmp));
 }
 
@@ -421,7 +431,15 @@ fpm_init(char* opt_file_name)
 #endif
 
   glb_launcher_string_list=NULL;
-  glb_win_app = create_app_safe();
+
+  if( glb_win_app == NULL)
+  {
+	/* First run */
+	fpm_ini_load();
+	glb_win_app = create_app_safe();
+  }
+  else
+	gtk_widget_hide(glb_win_app);
 
   if( opt_file_name )
     glb_filename = opt_file_name;
@@ -440,14 +458,16 @@ fpm_init(char* opt_file_name)
   	hostname, getpid());
 #endif
 
+//  glb_edit_data = g_malloc0(sizeof(fpm_data));
   glb_timer_click = g_timer_new();
   g_timer_start(glb_timer_click);
   glb_click_count=-1;
   glb_need_decrypt=FALSE;
   fpm_gpw_set_options(8, TRUE, TRUE, TRUE, FALSE, TRUE);
 
-  glb_clist_main = GTK_CLIST(lookup_widget (glb_win_app, "clist_main"));
-
+  glb_clist_main = GTK_TREE_VIEW(lookup_widget (glb_win_app, "clist_main"));
+  if(gtk_tree_view_get_model(glb_clist_main) == NULL) fpm_clist_init();
+  
   /* Lock the password file */
 
     lock_fpm_file(glb_filename);
@@ -474,21 +494,32 @@ fpm_quit(void)
   glb_quitting=TRUE;
   if (glb_dirty)
   {
-    gnome_question_dialog_modal("Do you want to save changes?",
-	 fpm_dialog_answer_save, NULL);
+    if (glb_fpm_ini->save_on_quit)
+	passfile_save(glb_filename);		
+    else
+	fpm_question(GTK_WINDOW(glb_win_app), _("Do you want to save changes?"),
+		 G_CALLBACK(fpm_dialog_answer_save));
   }
-  else
-  {
-    gtk_main_quit();
 
-  }
+  fpm_ini_save ();
+
+  gtk_main_quit();
 
 }
 
 void
-fpm_dialog_answer_save(gint reply, gpointer data)
+fpm_dialog_answer_edit(GtkWidget *widget, gint reply)
 {
-   if (reply==0)
+    if (reply == GTK_RESPONSE_YES) {
+	gtk_widget_destroy(glb_win_edit);
+	gtk_widget_set_sensitive(glb_win_app, TRUE);
+    }
+}
+
+void
+fpm_dialog_answer_save(GtkWidget *widget, gint reply)
+{
+   if (reply == GTK_RESPONSE_YES)
    {
      passfile_save(glb_filename);
    }
@@ -497,24 +528,23 @@ fpm_dialog_answer_save(gint reply, gpointer data)
 }
 
 void
-fpm_dialog_answer_delete(gint reply, gpointer data)
+fpm_dialog_answer_delete(GtkWidget *widget, gint reply)
 {
-  if (reply==0)
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+
+  if (reply == GTK_RESPONSE_YES)
   {
     glb_pass_list=g_list_remove(glb_pass_list, glb_edit_data);
-    gtk_clist_remove(glb_clist_main, glb_cur_row);
+    model = gtk_tree_view_get_model(glb_clist_main);
+    gtk_tree_model_get_iter_from_string(model, &iter, g_strdup_printf("%i",glb_cur_row));
+    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
     glb_num_row--;
-    if(glb_cur_row>=glb_num_row) glb_cur_row=glb_num_row;
-    if(glb_cur_row<0)
-    {
-      glb_edit_data=NULL;
-    }
-    else
-    {
-      glb_edit_data= (fpm_data*)
-        gtk_clist_get_row_data(glb_clist_main, glb_cur_row);
-    }
+
+    glb_edit_data = NULL;
     glb_dirty = TRUE;
+    if (glb_fpm_ini->save_on_delete)
+	passfile_save(glb_filename);
   }
 }
 
@@ -522,16 +552,13 @@ fpm_dialog_answer_delete(gint reply, gpointer data)
 void
 fpm_select(gchar* text, gboolean use_clipboard)
 {
-  GtkEntry* entry_hidden;
-  
+
   g_assert(text != NULL);
 
-  entry_hidden = GTK_ENTRY(lookup_widget(glb_win_app, "entry_hidden"));
-
-  gtk_entry_set_text(entry_hidden, text);
-  gtk_editable_select_region(GTK_EDITABLE(entry_hidden), 0, -1);
+  gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY), text, -1);
   if (use_clipboard)
-    gtk_editable_copy_clipboard(GTK_EDITABLE(entry_hidden));
+  gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD), text, -1);
+
 }
 
 void
@@ -601,15 +628,15 @@ fpm_jump(fpm_data* data)
 	    cmd=fpm_create_cmd_line(launcher->cmdline, data->arg,
 		 data->user, data->password);
 
-	    gnome_execute_shell(NULL, cmd);
+	    fpm_execute_shell(cmd);
     }
     else
     {
-    gnome_warning_dialog("This password's launcher command is undefined.\nPlease define it in the edit launcher preference screen.");
+    fpm_message(GTK_WINDOW(glb_win_app), _("This password's launcher command is undefined.\nPlease define it in the edit launcher preference screen."), GTK_MESSAGE_WARNING);
     }
   }
   else
-    gnome_warning_dialog("This password's launcher is undefined.\nPlease define it in the edit password item screen.");
+    fpm_message(GTK_WINDOW(glb_win_app), _("This password's launcher is undefined.\nPlease define it in the edit password item screen."), GTK_MESSAGE_WARNING);
 
 }
 
@@ -623,14 +650,15 @@ fpm_check_password()
 
   entry=GTK_ENTRY(lookup_widget(glb_win_misc, "entry_password"));
 
-  fpm_crypt_init(gtk_entry_get_text(entry));
+  fpm_crypt_init(gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1));
 
   fpm_decrypt_field(old_context, vstring_plain, vstring, 8);
   if (strcmp(vstring_plain, "FIGARO"))
   {
     gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
     label=GTK_LABEL(lookup_widget(glb_win_misc, "label_password_message"));
-    gtk_label_set_text(label, "Password incorrect.  Please try again.");
+    gtk_widget_show(GTK_WIDGET(label));
+    gtk_label_set_text(label, _("Password incorrect.  Please try again."));
     return;
   }
 
@@ -652,9 +680,11 @@ fpm_check_password()
     glb_last_category=g_strdup(FPM_ALL_CAT_MSG);
   }
 
+  glb_last_category = glb_fpm_ini->last_category;
+
   fpm_clist_populate_cat_list();
 
-
+  gtk_window_resize (GTK_WINDOW(glb_win_app), glb_fpm_ini->main_width, glb_fpm_ini->main_height);
 
   gtk_widget_show(glb_win_app);
   gtk_widget_destroy(glb_win_misc);
@@ -669,18 +699,18 @@ void fpm_set_password()
   byte *md5val, *prehash;
 
   entry=GTK_ENTRY(lookup_widget(glb_win_misc, "entry_cpw1"));
-  pw1=gtk_entry_get_text(entry); 
+  pw1 = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
   entry=GTK_ENTRY(lookup_widget(glb_win_misc, "entry_cpw2"));
-  pw2=gtk_entry_get_text(entry); 
+  pw2 = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
   
   if (strcmp(pw1, pw2))
   {
-    gnome_error_dialog("Sorry, the passwords do not match.");
+    fpm_message(GTK_WINDOW(glb_win_misc), _("Sorry, the passwords do not match."), GTK_MESSAGE_ERROR);
     return;
   }
   if (strlen(pw1)<4)
   {
-    gnome_error_dialog("Sorry, your password is too short.");
+    fpm_message(GTK_WINDOW(glb_win_misc), _("Sorry, your password is too short."), GTK_MESSAGE_ERROR);
     return;
   }
 
@@ -703,8 +733,8 @@ void fpm_set_password()
   {
     /* We are chaning the password.*/
     new_salt=get_new_salt();
-    prehash=g_strdup_printf("%s%s", new_salt, pw1);
-    md5val=md5(prehash, strlen(prehash));
+    prehash=(guchar *) g_strdup_printf("%s%s", new_salt, pw1);
+    md5val=md5(prehash, strlen((gchar *) prehash));
     fpm_setkey(new_context, md5val, 16);
     g_free(md5val);
     gtk_widget_destroy(glb_win_misc);
@@ -812,7 +842,7 @@ gchar* fpm_create_cmd_line(gchar* cmd, gchar* arg, gchar* user, gchar* pass)
       g_free(cleartext);
     }
     else
-      result=g_strconcat(tmp, "$", cmd_arr[i]);
+      result=g_strconcat(tmp, "$", cmd_arr[i], NULL);
     memset(tmp, 0, strlen(tmp));
     g_free(tmp);
     i++;
@@ -827,3 +857,244 @@ void fpm_debug(gchar* msg)
   printf("Debug %s:%d:%d\n", msg, (glb_launcher_list==NULL), (glb_launcher_string_list==NULL));
 
 }
+                            
+void fpm_message(GtkWindow* win, gchar* message, GtkMessageType message_type)
+{
+    GtkWidget *dw;
+    dw = gtk_message_dialog_new (GTK_WINDOW(win),
+                              GTK_DIALOG_DESTROY_WITH_PARENT,
+                              message_type,
+                              GTK_BUTTONS_CLOSE, 
+                              message);
+    gtk_window_set_title (GTK_WINDOW(dw), _("Information"));
+    gtk_dialog_run (GTK_DIALOG (dw));
+    gtk_widget_destroy (dw);
+}
+
+void fpm_question(GtkWindow* win, gchar* message, GCallback callback)
+{
+    GtkWidget *dw;
+    dw = gtk_message_dialog_new (GTK_WINDOW(win),
+                        GTK_DIALOG_DESTROY_WITH_PARENT,
+                        GTK_MESSAGE_QUESTION,
+			GTK_BUTTONS_YES_NO,
+                        message);
+    gtk_window_set_title (GTK_WINDOW(dw),_("Question"));
+
+    g_signal_connect (dw,
+            "response", 
+	     G_CALLBACK (callback),
+    	     NULL);
+    	     
+    gint result = gtk_dialog_run (GTK_DIALOG (dw));
+    if(result != GTK_RESPONSE_NONE) gtk_widget_destroy (dw);
+}
+
+void fpm_ini_load()
+{
+    GKeyFile *keyfile;
+    GError *error = NULL;
+    gsize count;
+    gint i;
+    gint *columns;
+    
+    glb_fpm_ini = g_malloc0(sizeof(fpm_ini));
+
+    gchar *fpm_ini_file = g_build_filename (g_get_home_dir(), ".fpm", "fpm.ini", NULL);
+
+    keyfile = g_key_file_new ();
+    if (!g_key_file_load_from_file (keyfile, fpm_ini_file, G_KEY_FILE_NONE, NULL))
+    {
+	/* No ini file found -> set defaults and save */
+        glb_fpm_ini->save_on_add = TRUE;
+        glb_fpm_ini->save_on_change = TRUE;
+        glb_fpm_ini->save_on_delete = FALSE;
+        glb_fpm_ini->save_on_quit = FALSE;
+        glb_fpm_ini->create_backup = TRUE;
+        glb_fpm_ini->number_backup_files = 5;
+	glb_fpm_ini->main_width = 500;	
+	glb_fpm_ini->main_height = 350;
+	glb_fpm_ini->last_category = FPM_ALL_CAT_MSG;
+	glb_last_category = FPM_ALL_CAT_MSG;
+	glb_fpm_ini->search_in_title = TRUE;
+	glb_fpm_ini->search_in_url = FALSE;
+	glb_fpm_ini->search_in_user = FALSE;
+	glb_fpm_ini->search_in_notes = FALSE;
+	glb_fpm_ini->search_match_case = FALSE;
+	glb_fpm_ini->search_limit_category = TRUE;
+	fpm_ini_save();
+    } else {
+	glb_fpm_ini->save_on_add = g_key_file_get_boolean (keyfile, "general", "save_on_add", NULL);
+        glb_fpm_ini->save_on_delete = g_key_file_get_boolean (keyfile, "general", "save_on_delete", NULL);
+        glb_fpm_ini->save_on_quit = g_key_file_get_boolean (keyfile, "general", "save_on_quit", NULL);
+        glb_fpm_ini->save_on_change = g_key_file_get_boolean (keyfile, "general", "save_on_change", NULL);
+        glb_fpm_ini->create_backup = g_key_file_get_boolean (keyfile, "general", "create_backup", NULL);
+        glb_fpm_ini->number_backup_files = g_key_file_get_integer (keyfile, "general", "number_backup_files", NULL);
+
+        glb_fpm_ini->last_category = g_key_file_get_string (keyfile, "general", "last_category", &error);
+	if (error != NULL)
+	    glb_fpm_ini->last_category = FPM_ALL_CAT_MSG;
+	    
+	error = NULL;
+	glb_fpm_ini->main_width = g_key_file_get_integer (keyfile, "general", "main_width", &error);
+	if (error != NULL)
+	    glb_fpm_ini->main_width = 500;	
+
+	error = NULL;
+	glb_fpm_ini->main_height = g_key_file_get_integer (keyfile, "general", "main_height", &error);
+	if (error != NULL)
+	    glb_fpm_ini->main_height = 350;
+
+	error = NULL;
+	columns = g_key_file_get_integer_list (keyfile, "general", "columns_width", &count, &error);
+	if (error != NULL || count != FPM_NUM_COL || columns == NULL) {
+	    for (i=0; i< FPM_NUM_COL; i++) 
+		glb_fpm_ini->columns_width[i] = 150;
+	} else {
+	    memcpy(glb_fpm_ini->columns_width, columns, sizeof(glb_fpm_ini->columns_width));
+	    g_free(columns);
+	}    
+
+	error = NULL;
+	glb_fpm_ini->search_in_title = g_key_file_get_boolean (keyfile, "search", "search_in_title", &error);
+	if (error != NULL)
+	    glb_fpm_ini->search_in_title = TRUE;
+
+	glb_fpm_ini->search_in_url = g_key_file_get_boolean (keyfile, "search", "search_in_url", NULL);
+	glb_fpm_ini->search_in_user = g_key_file_get_boolean (keyfile, "search", "search_in_user", NULL);
+	glb_fpm_ini->search_in_notes = g_key_file_get_boolean (keyfile, "search", "search_in_notes", NULL);
+	glb_fpm_ini->search_match_case = g_key_file_get_boolean (keyfile, "search", "search_match_case", NULL);
+
+	error = NULL;
+	glb_fpm_ini->search_limit_category = g_key_file_get_boolean (keyfile, "search", "search_limit_category", &error);
+	if (error != NULL)
+	    glb_fpm_ini->search_limit_category = TRUE;
+
+	glb_last_category = glb_fpm_ini->last_category;
+    }
+
+    g_key_file_free (keyfile);
+}
+
+void fpm_ini_save()
+{
+    GKeyFile *keyfile;
+    GtkTreeViewColumn *column;
+    gint width, height;
+    gint i;
+    
+    if (glb_win_app == NULL) {
+	width = 500;
+	height = 350;
+	for (i=0; i< FPM_NUM_COL; i++) 
+	    glb_fpm_ini->columns_width[i] = 150;
+    } else {
+	gtk_window_get_size (GTK_WINDOW(glb_win_app), &width, &height);
+	    
+	for (i=0; i< FPM_NUM_COL; i++) {
+	    column = gtk_tree_view_get_column (glb_clist_main, i);
+	    glb_fpm_ini->columns_width[i] = gtk_tree_view_column_get_width (column);
+	}
+    }
+
+    gchar *fpm_ini_file = g_build_filename (g_get_home_dir(), ".fpm", "fpm.ini", NULL);
+    
+    keyfile = g_key_file_new ();
+
+    g_key_file_set_boolean (keyfile, "general", "save_on_add", glb_fpm_ini->save_on_add);
+    g_key_file_set_boolean (keyfile, "general", "save_on_change", glb_fpm_ini->save_on_change);
+    g_key_file_set_boolean (keyfile, "general", "save_on_delete", glb_fpm_ini->save_on_delete);
+    g_key_file_set_boolean (keyfile, "general", "save_on_quit", glb_fpm_ini->save_on_quit);
+    g_key_file_set_boolean (keyfile, "general", "create_backup", glb_fpm_ini->create_backup);
+    g_key_file_set_integer (keyfile, "general", "number_backup_files", glb_fpm_ini->number_backup_files);
+    g_key_file_set_string (keyfile, "general", "last_category", glb_last_category);
+    g_key_file_set_integer (keyfile, "general", "main_width", width);
+    g_key_file_set_integer (keyfile, "general", "main_height", height);
+    g_key_file_set_integer_list (keyfile, "general", "columns_width", glb_fpm_ini->columns_width, 3);
+
+    g_key_file_set_boolean (keyfile, "search", "search_in_title", glb_fpm_ini->search_in_title);
+    g_key_file_set_boolean (keyfile, "search", "search_in_url", glb_fpm_ini->search_in_url);
+    g_key_file_set_boolean (keyfile, "search", "search_in_user", glb_fpm_ini->search_in_user);
+    g_key_file_set_boolean (keyfile, "search", "search_in_notes", glb_fpm_ini->search_in_notes);
+    g_key_file_set_boolean (keyfile, "search", "search_match_case", glb_fpm_ini->search_match_case);
+    g_key_file_set_boolean (keyfile, "search", "search_limit_category", glb_fpm_ini->search_limit_category);
+
+    gchar *save_data = g_key_file_to_data (keyfile, NULL, NULL);
+
+    g_file_set_contents (fpm_ini_file, save_data, -1, NULL);
+    
+    g_key_file_free (keyfile);
+}
+
+void fpm_search(gchar *search_text)
+{
+    fpm_data *data;
+    GList *list;
+    GtkTreeIter iter;
+    GtkTreeModel *list_store;
+    gchar *title, *url, *user, *notes;
+
+    if (!glb_fpm_ini->search_match_case)
+	search_text = g_utf8_casefold(search_text, -1);
+
+    list_store = gtk_tree_view_get_model (glb_clist_main);
+    gtk_list_store_clear(GTK_LIST_STORE(list_store));
+
+    list = g_list_first(glb_pass_list);
+    while (list != NULL)
+    {
+	data = list->data;
+        title = data->title;
+        url = data->arg;
+	user = data->user;
+    	notes = data->notes;
+
+        if ((!strcmp(glb_last_category, data->category) && glb_fpm_ini->search_limit_category)
+	    || ((!strcmp(glb_last_category, FPM_ALL_CAT_MSG)
+	    || !glb_fpm_ini->search_limit_category)))
+	{
+
+    	    if (!glb_fpm_ini->search_match_case) {
+		title = g_utf8_casefold(title, -1);
+		url = g_utf8_casefold(url, -1);
+		user = g_utf8_casefold(user, -1);
+		notes = g_utf8_casefold(notes, -1);
+	    }
+
+	    if ((strstr(title, search_text) && glb_fpm_ini->search_in_title)
+		|| (strstr(url, search_text) && glb_fpm_ini->search_in_url)
+		|| (strstr(user, search_text) && glb_fpm_ini->search_in_user)
+		|| (strstr(notes, search_text) && glb_fpm_ini->search_in_notes))
+	    {
+
+        	gtk_list_store_append (GTK_LIST_STORE(list_store), &iter);
+
+		gtk_list_store_set(GTK_LIST_STORE(list_store), &iter,
+	    	    FPM_COL_TITLE, data->title,
+    		    FPM_COL_URL, data->arg,
+	    	    FPM_COL_USER, data->user,
+	    	    FPM_COL_DATA_POINTER, data,
+	    	    -1);
+	    }
+	}
+	list=g_list_next(list);
+    }
+}
+
+void fpm_execute_shell(gchar *cmd)
+{
+    gchar *execute;
+    int ret;
+    
+    execute = g_strjoin(NULL, cmd, " &", NULL);
+    ret = system(execute);
+}
+
+void fpm_statusbar_push(gchar *message)
+{
+    GtkStatusbar *statusbar;
+    
+    statusbar = GTK_STATUSBAR(lookup_widget (glb_win_app, "statusbar"));
+    gtk_statusbar_pop(statusbar, 1);
+    gtk_statusbar_push(statusbar, 1, message);
+}
