@@ -18,13 +18,14 @@
  * fpm.h
  */
 
-#include <gnome.h>
+#include <gtk/gtk.h>
 
 /* CONSTANTS */
 #define FPM_NUM_COL 	 3	 /* Number of columns in main CList */
 #define FPM_COL_TITLE	 0
 #define FPM_COL_URL	 1
 #define FPM_COL_USER	 2
+#define FPM_COL_DATA_POINTER	 3
 
 #define FPM_MAX_DOUBLE_CLICK_TIME 0.8 /* In seconds */
 
@@ -33,13 +34,13 @@
 
 #define FPM_BACKUP_NUM 20
 
-#define DISPLAY_VERSION 	"0.58"
-#define FULL_VERSION 		"00.58.00"
+#define DISPLAY_VERSION 	"0.71"
+#define FULL_VERSION 		"00.71.00"
 #define MIN_VERSION		"00.58.00"
 #define ENCRYPT_VERSION		"00.23.00"
 
-#define FPM_ALL_CAT_MSG		"<ALL CATEGORIES>"
-#define FPM_DEFAULT_CAT_MSG	"<DEFAULTS>"
+#define FPM_ALL_CAT_MSG		_("<ALL CATEGORIES>")
+#define FPM_DEFAULT_CAT_MSG	_("<DEFAULTS>")
 
 /* STRUCTURES */
 typedef struct fpm_data_struct
@@ -63,8 +64,28 @@ typedef struct fpm_launcher_struct
   gint copy_password;
 } fpm_launcher;
 
+typedef struct fpm_ini_struct
+{
+    gboolean save_on_quit;
+    gboolean save_on_change;
+    gboolean save_on_add;
+    gboolean save_on_delete;
+    gboolean create_backup;
+    gint number_backup_files;
+    gchar *last_category;
+    gint main_width;
+    gint main_height;
+    gint columns_width[3];
+    gboolean search_match_case;
+    gboolean search_in_title;
+    gboolean search_in_url;
+    gboolean search_in_user;
+    gboolean search_in_notes;
+    gboolean search_limit_category;
+} fpm_ini;
+
 extern GtkWidget* glb_win_app; 	 /* The main window for the application   */
-extern GtkCList* glb_clist_main; /* Password CList on main window         */
+extern GtkTreeView* glb_clist_main; /* Password CList on main window         */
 
 extern GtkWidget* glb_win_misc;  /* Misc window, used for dialogs         */
 
@@ -77,6 +98,8 @@ extern gchar* glb_filename;	 /* The location of the pasword file.     */
 extern gboolean glb_dirty;	 /* Have changes been made?		  */
 extern GTimer* glb_timer_click;  /* Timer used to check for double clicks */
 extern gint glb_click_count;     /* Click count used for double clicks    */
+extern gboolean glb_click_btn;   /* Right button clicked -> context menu  */
+
 
 extern gchar* new_salt;          /* Salt to use on next save.             */
 extern gchar* old_salt;          /* Salt to used on last save.            */
@@ -99,8 +122,7 @@ extern gboolean glb_using_defaults;
 
 extern gchar* glb_last_category;
 
-
-
+extern fpm_ini* glb_fpm_ini;	/* Store FPM settings. */
 
 void
 fpm_new_passitem(GtkWidget** win_edit_ptr, fpm_data** data_ptr);  
@@ -124,10 +146,10 @@ void
 fpm_init(char* opt_file_name);
 
 void
-fpm_dialog_answer_delete(gint reply, gpointer data);
+fpm_dialog_answer_delete(GtkWidget* widget, gint reply);
 
 void
-fpm_dialog_answer_save(gint reply, gpointer data);
+fpm_dialog_answer_save(GtkWidget* widget, gint reply);
 
 void
 fpm_select(gchar* text, gboolean use_clipboard);
@@ -147,7 +169,8 @@ fpm_check_password(void);
 void
 fpm_set_password(void);
 
-void fpm_clear_list(void);
+void
+ fpm_clear_list(void);
 
 void
 fpm_clist_create_view(gchar* category); /* In fpm_clist.c */
@@ -155,6 +178,8 @@ fpm_clist_create_view(gchar* category); /* In fpm_clist.c */
 void
 fpm_clist_populate_cat_list(void);
 
+void
+fpm_clist_init(void);
 
 void
 fpm_check_view(gchar * category);
@@ -162,7 +187,8 @@ fpm_check_view(gchar * category);
 void
 fpm_create_category_list(gint edit_flag);
 
-gchar* fpm_create_cmd_line(gchar* cmd, gchar* arg, gchar* user, gchar* pass);
+gchar*
+fpm_create_cmd_line(gchar* cmd, gchar* arg, gchar* user, gchar* pass);
 
 void fpm_init_launchers(void);
 
@@ -175,4 +201,21 @@ void fpm_prefs_save_launcher(void);
 void fpm_debug(gchar* msg);
 
 void lock_fpm_file(gchar* glb_filename);
+
 void unlock_fpm_file(void);
+
+void fpm_dialog_answer_edit(GtkWidget* widget, gint reply);
+
+void fpm_message(GtkWindow* win, gchar* message, GtkMessageType message_type);
+
+void fpm_question(GtkWindow* win, gchar* message, GCallback callback);
+
+void fpm_ini_load(void);
+
+void fpm_ini_save(void);
+
+void fpm_search(gchar *search_text);
+
+void fpm_execute_shell(gchar *cmd);
+
+void fpm_statusbar_push(gchar *message);
