@@ -20,7 +20,7 @@
  * to compile.
  */
 
-#include <gnome.h>
+#include <gtk/gtk.h>
 
 /* From types.h: */
 typedef guint32 u32; /* GPG used complex method to ensure 32 bit unsigned int.
