@@ -17,9 +17,12 @@
  *
  * passfile.c -- Routines to save and load XML files with FPM data.
  */
-#include <gnome.h>
+
+#include <gtk/gtk.h>
 #include <libxml/parser.h>
 #include <libxml/tree.h>
+#include <string.h>
+
 #include "fpm.h"
 #include "passfile.h"
 #include "fpm_crypt.h"
@@ -50,10 +53,10 @@ passfile_update_key()
 
 
 
-  // printf("XXX passfile_update_key being called\n");
+//   printf("XXX passfile_update_key being called\n");
   if (old_context!=new_context)
   {
-    // printf("XXX run through the list\n");
+//     printf("XXX run through the list\n");
     list=g_list_first(glb_pass_list);
     while(list!=NULL)
     {
@@ -61,7 +64,7 @@ passfile_update_key()
       ix = strlen(data->password) / 2;
       fpm_decrypt_field(old_context, plaintext,
 	 data->password, ix);
-      // printf("XXX passwd = %s\n", plaintext);
+//       printf("XXX passwd = %s\n", plaintext);
       fpm_encrypt_field(new_context, data->password,
 	 plaintext, FPM_PASSWORD_LEN);
       list = g_list_next(list);
@@ -71,6 +74,7 @@ passfile_update_key()
     bcopy(new_context, old_context , blowfish_contextsize());
     old_salt = new_salt;
   }
+//   printf("XXX passfile_update_key exit\n");
 }
 
 static void
@@ -85,7 +89,7 @@ move_backup(gchar* file_name)
   gint i;
 
 
-  for(i=FPM_BACKUP_NUM;i>0;i--)
+  for(i=glb_fpm_ini->number_backup_files;i>0;i--)
   {
     g_snprintf(file2, 512, "%s.%02d", file_name, i);
     if (i>1)
@@ -99,22 +103,22 @@ move_backup(gchar* file_name)
 }
 
 static void
-passfile_upd_attr(xmlNodePtr node, char* cond, char** data_ptr)
+passfile_upd_attr(xmlNodePtr node, char *cond, char** data_ptr)
 {
   
-  if (!strcmp(node->name, cond))
+  if (!strcmp((char *)node->name, cond))
   {
     g_free(*data_ptr);
     if(node->xmlChildrenNode != NULL)
-      *data_ptr=g_strdup(node->xmlChildrenNode->content);
+      *data_ptr=g_strdup((char *)node->xmlChildrenNode->content);
     else
       *data_ptr=g_strdup("");
   }
 }
 
-static void new_leaf(xmlDocPtr doc, xmlNodePtr item, const char *name, char *text)
+static void new_leaf(xmlDocPtr doc, xmlNodePtr item, const xmlChar *name, xmlChar *text)
 {
-  char* tmp;
+  xmlChar *tmp;
 
   tmp = xmlEncodeEntitiesReentrant(doc, text);
   xmlNewChild(item, NULL, name, tmp);
@@ -122,12 +126,12 @@ static void new_leaf(xmlDocPtr doc, xmlNodePtr item, const char *name, char *tex
 }
   
 static void new_leaf_encrypt
-	(xmlDocPtr doc, xmlNodePtr item, const char *name, char *text)
+	(xmlDocPtr doc, xmlNodePtr item, const xmlChar *name, char *text)
 {
-  gchar* cipher_text;
+  char *cipher_text;
   // printf("XXX calling new_leaf_encrypt: %s\n", text);
   cipher_text=fpm_encrypt_field_var(new_context, text);
-  new_leaf(doc, item, name, cipher_text);
+  new_leaf(doc, item, name, (xmlChar *)cipher_text);
   g_free(cipher_text);
 }
 
@@ -145,63 +149,63 @@ passfile_save(gchar* file_name)
   gchar* tmp;
   GList * list;
   gint i;
+  gchar *dia_str;
 
-  move_backup(file_name);
+  if (glb_fpm_ini->create_backup) 
+    move_backup(file_name);
   passfile_update_key();
 
   vstring_cipher=g_malloc0(17);
   vstring_plain=g_malloc0(9);
 
-  doc = xmlNewDoc("1.0");
-  doc->xmlRootNode = xmlNewDocNode(doc, NULL, "FPM", NULL);
-  xmlSetProp(doc->xmlRootNode, "full_version", FULL_VERSION);
-  xmlSetProp(doc->xmlRootNode, "min_version", MIN_VERSION);
-  xmlSetProp(doc->xmlRootNode, "display_version", DISPLAY_VERSION);
-  item = xmlNewChild(doc->xmlRootNode, NULL, "KeyInfo", NULL);
-  xmlSetProp(item, "salt", new_salt);
+  doc = xmlNewDoc((xmlChar *)"1.0");
+  doc->xmlRootNode = xmlNewDocNode(doc, NULL, (xmlChar *)"FPM", NULL);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"full_version", (xmlChar *)FULL_VERSION);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"min_version", (xmlChar *)MIN_VERSION);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"display_version", (xmlChar *)DISPLAY_VERSION);
+  item = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"KeyInfo", NULL);
+  xmlSetProp(item, (xmlChar *)"salt", (xmlChar *)new_salt);
   strncpy(vstring_plain, "FIGARO", 8);
   fpm_encrypt_field(new_context, vstring_cipher, vstring_plain, 8);
-  xmlSetProp(item, "vstring", vstring_cipher);
+  xmlSetProp(item, (xmlChar *)"vstring", (xmlChar *)vstring_cipher);
 
   fpm_decrypt_field(new_context, vstring_plain, vstring_cipher, 8);
 
-
-
-  nodelist = xmlNewChild(doc->xmlRootNode, NULL, "LauncherList", NULL);
+  nodelist = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"LauncherList", NULL);
   list = g_list_first(glb_launcher_list);
   
   while (list!=NULL)
   {
     launcher=list->data;
-    item=xmlNewChild(nodelist, NULL, "LauncerItem", NULL);
-    new_leaf(doc, item, "title", launcher->title);
-    new_leaf(doc, item, "cmdline", launcher->cmdline);
+    item=xmlNewChild(nodelist, NULL, (xmlChar *)"LauncerItem", NULL);
+    new_leaf(doc, item, (xmlChar *)"title", (xmlChar *)launcher->title);
+    new_leaf(doc, item, (xmlChar *)"cmdline", (xmlChar *)launcher->cmdline);
     tmp=g_strdup_printf("%d", launcher->copy_user);
-    new_leaf(doc, item, "copy_user", tmp);
+    new_leaf(doc, item, (xmlChar *)"copy_user", (xmlChar *)tmp);
     g_free(tmp);
     tmp=g_strdup_printf("%d", launcher->copy_password);
-    new_leaf(doc, item, "copy_password", tmp);
+    new_leaf(doc, item, (xmlChar *)"copy_password", (xmlChar *)tmp);
     g_free(tmp);
 
     list=g_list_next(list);
   }
-  
-  nodelist = xmlNewChild(doc->xmlRootNode, NULL, "PasswordList", NULL);
+
+  nodelist = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"PasswordList", NULL);
   list = g_list_first(glb_pass_list);
   i=0;
   while (list!=NULL)
   {
     data = list->data;
-    item = xmlNewChild(nodelist, NULL, "PasswordItem", NULL);
-    new_leaf_encrypt(doc, item, "title", data->title);
-    new_leaf_encrypt(doc, item, "user", data->user);
-    new_leaf_encrypt(doc, item, "url", data->arg);
-    new_leaf(doc, item, "password", data->password);
-    new_leaf_encrypt(doc, item, "notes", data->notes);
-    new_leaf_encrypt(doc, item, "category", data->category);
-    new_leaf_encrypt(doc, item, "launcher", data->launcher);
+    item = xmlNewChild(nodelist, NULL, (xmlChar *)"PasswordItem", NULL);
+    new_leaf_encrypt(doc, item, (xmlChar *)"title", data->title);
+    new_leaf_encrypt(doc, item, (xmlChar *)"user", data->user);
+    new_leaf_encrypt(doc, item, (xmlChar *)"url", data->arg);
+    new_leaf(doc, item, (xmlChar *)"password", (xmlChar *)data->password);
+    new_leaf_encrypt(doc, item, (xmlChar *)"notes", data->notes);
+    new_leaf_encrypt(doc, item, (xmlChar *)"category", data->category);
+    new_leaf_encrypt(doc, item, (xmlChar *)"launcher", data->launcher);
   
-    if (data->default_list) xmlNewChild(item, NULL, "default", NULL);
+    if (data->default_list) xmlNewChild(item, NULL, (xmlChar *)"default", NULL);
 
     list=g_list_next(list);
     i++;
@@ -211,10 +215,13 @@ passfile_save(gchar* file_name)
   xmlDocDump(file, doc);
   fclose(file);
   cmd = g_strdup_printf("chmod 0600 %s", file_name);
-  gnome_execute_shell(NULL, cmd);
+  fpm_execute_shell(cmd);
   g_free(cmd);
 
-  printf("Saved %d passwords.\n", i);
+  dia_str = g_strdup_printf(_("Saved %d password(s)."), i);
+  fpm_statusbar_push(dia_str);
+
+  printf("Saved %d password(s).\n", i);
   glb_dirty = FALSE;
   g_free(vstring_plain);
   g_free(vstring_cipher);
@@ -255,41 +262,39 @@ passfile_export(gchar* file_name)
   vstring_cipher=g_malloc0(17);
   vstring_plain=g_malloc0(9);
 
-  doc = xmlNewDoc("1.0");
-  doc->xmlRootNode = xmlNewDocNode(doc, NULL, "FPM", NULL);
-  xmlSetProp(doc->xmlRootNode, "full_version", FULL_VERSION);
-  xmlSetProp(doc->xmlRootNode, "min_version", MIN_VERSION);
-  xmlSetProp(doc->xmlRootNode, "display_version", DISPLAY_VERSION);
-  item = xmlNewChild(doc->xmlRootNode, NULL, "KeyInfo", NULL);
-  xmlSetProp(item, "salt", new_salt);
+  doc = xmlNewDoc((xmlChar *)"1.0");
+  doc->xmlRootNode = xmlNewDocNode(doc, NULL, (xmlChar *)"FPM", NULL);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"full_version", (xmlChar *)FULL_VERSION);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"min_version", (xmlChar *)MIN_VERSION);
+  xmlSetProp(doc->xmlRootNode, (xmlChar *)"display_version", (xmlChar *)DISPLAY_VERSION);
+  item = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"KeyInfo", NULL);
+  xmlSetProp(item, (xmlChar *)"salt", (xmlChar *)new_salt);
   strncpy(vstring_plain, "FIGARO", 8);
   fpm_encrypt_field(new_context, vstring_cipher, vstring_plain, 8);
-  xmlSetProp(item, "vstring", vstring_cipher);
+  xmlSetProp(item, (xmlChar *)"vstring", (xmlChar *)vstring_cipher);
 
   fpm_decrypt_field(new_context, vstring_plain, vstring_cipher, 8);
 
-
-
-  nodelist = xmlNewChild(doc->xmlRootNode, NULL, "LauncherList", NULL);
+  nodelist = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"LauncherList", NULL);
   list = g_list_first(glb_launcher_list);
   
   while (list!=NULL)
   {
     launcher=list->data;
-    item=xmlNewChild(nodelist, NULL, "LauncerItem", NULL);
-    new_leaf(doc, item, "title", launcher->title);
-    new_leaf(doc, item, "cmdline", launcher->cmdline);
+    item=xmlNewChild(nodelist, NULL, (xmlChar *)"LauncerItem", NULL);
+    new_leaf(doc, item, (xmlChar *)"title", (xmlChar *)launcher->title);
+    new_leaf(doc, item, (xmlChar *)"cmdline", (xmlChar *)launcher->cmdline);
     tmp=g_strdup_printf("%d", launcher->copy_user);
-    new_leaf(doc, item, "copy_user", tmp);
+    new_leaf(doc, item, (xmlChar *)"copy_user", (xmlChar *)tmp);
     g_free(tmp);
     tmp=g_strdup_printf("%d", launcher->copy_password);
-    new_leaf(doc, item, "copy_password", tmp);
+    new_leaf(doc, item, (xmlChar *)"copy_password", (xmlChar *)tmp);
     g_free(tmp);
 
     list=g_list_next(list);
   }
   
-  nodelist = xmlNewChild(doc->xmlRootNode, NULL, "PasswordList", NULL);
+  nodelist = xmlNewChild(doc->xmlRootNode, NULL, (xmlChar *)"PasswordList", NULL);
   list = g_list_first(glb_pass_list);
   i=0;
   if ( ( fp = fopen(new_file_name2, "w")) == NULL )
@@ -300,18 +305,18 @@ passfile_export(gchar* file_name)
   while (list!=NULL)
   {
     data = list->data;
-    item = xmlNewChild(nodelist, NULL, "PasswordItem", NULL);
-    new_leaf(doc, item, "title", data->title);
-    new_leaf(doc, item, "user", data->user);
-    new_leaf(doc, item, "url", data->arg);
+    item = xmlNewChild(nodelist, NULL, (xmlChar *)"PasswordItem", NULL);
+    new_leaf(doc, item, (xmlChar *)"title", (xmlChar *)data->title);
+    new_leaf(doc, item, (xmlChar *)"user", (xmlChar *)data->user);
+    new_leaf(doc, item, (xmlChar *)"url", (xmlChar *)data->arg);
     fpm_decrypt_field(old_context, plaintext, data->password, FPM_PASSWORD_LEN);
-    new_leaf(doc, item, "password", plaintext);
-    new_leaf(doc, item, "notes", data->notes);
-    new_leaf(doc, item, "category", data->category);
-    new_leaf(doc, item, "launcher", data->launcher);
+    new_leaf(doc, item, (xmlChar *)"password", (xmlChar *)plaintext);
+    new_leaf(doc, item, (xmlChar *)"notes", (xmlChar *)data->notes);
+    new_leaf(doc, item, (xmlChar *)"category", (xmlChar *)data->category);
+    new_leaf(doc, item, (xmlChar *)"launcher", (xmlChar *)data->launcher);
     fprintf(fp, "title=%s\nuser=%s\npasswd=%s\nurl=%s\nnotes=%s\n\n",  data->title, data->user, plaintext, data->arg, data->notes);
   
-    if (data->default_list) xmlNewChild(item, NULL, "default", NULL);
+    if (data->default_list) xmlNewChild(item, NULL, (xmlChar *)"default", NULL);
 
     list=g_list_next(list);
     i++;
@@ -322,18 +327,18 @@ passfile_export(gchar* file_name)
   xmlDocDump(file, doc);
   fclose(file);
   cmd = g_strdup_printf("chmod 0600 %s", new_file_name);
-  gnome_execute_shell(NULL, cmd);
+  fpm_execute_shell(cmd);
   g_free(cmd);
   cmd = g_strdup_printf("chmod 0600 %s", new_file_name2);
-  gnome_execute_shell(NULL, cmd);
+ fpm_execute_shell(cmd);
   g_free(cmd);
 
   printf("Saved %d passwords.\n", i);
   glb_dirty = FALSE;
   g_free(vstring_plain);
   g_free(vstring_cipher);
-  sprintf(dia_str, "Exported clear text passwords to\n %s and\n %s", new_file_name, new_file_name2);
-  gnome_ok_dialog( dia_str );
+  sprintf(dia_str, _("Exported clear text passwords to\n %s and\n %s"), new_file_name, new_file_name2);
+  fpm_message(GTK_WINDOW(glb_win_app), dia_str, GTK_MESSAGE_INFO);
   g_free(new_file_name);
   g_free(new_file_name2);
   g_free(dia_str);
@@ -351,6 +356,9 @@ passfile_load(gchar* file_name)
   fpm_data* data;
   fpm_launcher* launcher;
   gint i;
+  //GtkStatusbar *entry;
+  gchar *dia_str;
+
 
   /* Start from scratch */
   if(glb_pass_list != NULL) fpm_clear_list();
@@ -370,17 +378,18 @@ passfile_load(gchar* file_name)
     return(-1);
   }
 
-  /* Check if document is one of ours */
-  g_return_val_if_fail(!strcmp(doc->xmlRootNode->name, "FPM"), -2);
 
-  file_version = xmlGetProp(doc->xmlRootNode, "min_version");
+  /* Check if document is one of ours */
+  g_return_val_if_fail(!xmlStrcmp(doc->xmlRootNode->name, (xmlChar *)"FPM"), -2);
+
+  file_version = (char *)xmlGetProp(doc->xmlRootNode, (xmlChar *)"min_version");
   if (strcmp(file_version, FULL_VERSION) > 0)
   {
     g_error("Sorry, the password file cannot be read because it uses a future file format.  Please download the latest version of FPM and try again.\n");
     gtk_main_quit(); 
   }
 
-  file_version = xmlGetProp(doc->xmlRootNode, "full_version");
+  file_version = (char *)xmlGetProp(doc->xmlRootNode, (xmlChar *)"full_version");
  
 /* Current versions of FPM encrypt all fields.  Fields other than the 
  * password are decrypted when the program reads the password file, 
@@ -392,23 +401,24 @@ passfile_load(gchar* file_name)
  * only encrypted passwords.
  */
  
+
   glb_need_decrypt = (strcmp(file_version, ENCRYPT_VERSION) >= 0);
 
   list=doc->xmlRootNode->xmlChildrenNode;
-  old_salt = xmlGetProp(list, "salt");
-  vstring = xmlGetProp(list, "vstring");
+  old_salt = (char *)xmlGetProp(list, (xmlChar *)"salt");
+  vstring = (char *)xmlGetProp(list, (xmlChar *)"vstring");
   new_salt = get_new_salt();
   glb_using_defaults=FALSE;
   
   list=list->next;
 
-  if (list==NULL || ( strcmp(list->name, "PasswordList") && strcmp(list->name, "LauncherList") ))
+  if (list==NULL || ( strcmp((char *)list->name, "PasswordList") && strcmp((char *)list->name, "LauncherList") ))
   {
     g_error("Invalid password file.");
     gtk_main_quit();
   }
 
-  if(!strcmp(list->name, "LauncherList"))
+  if(!strcmp((char *)list->name, "LauncherList"))
   {
     printf("Loading launchers...\n");
     glb_launcher_list=NULL;
@@ -423,25 +433,25 @@ passfile_load(gchar* file_name)
       attr=item->xmlChildrenNode;
       while(attr!=NULL)
       {
-	if(!strcmp(attr->name, "title") && attr->xmlChildrenNode && attr->xmlChildrenNode->content)
+	if(!strcmp((char *)attr->name, "title") && attr->xmlChildrenNode && attr->xmlChildrenNode->content)
 	{
 	  g_free(launcher->title);
-	  launcher->title=g_strdup(attr->xmlChildrenNode->content);
+	  launcher->title=g_strdup((char *)attr->xmlChildrenNode->content);
 	}
-	if(!strcmp(attr->name, "cmdline") && attr->xmlChildrenNode && attr->xmlChildrenNode->content)
+	if(!strcmp((char *)attr->name, "cmdline") && attr->xmlChildrenNode && attr->xmlChildrenNode->content)
 	{
 	  g_free(launcher->cmdline);
-	  launcher->cmdline=g_strdup(attr->xmlChildrenNode->content);
+	  launcher->cmdline=g_strdup((char *)attr->xmlChildrenNode->content);
 	}
-	if(!strcmp(attr->name, "copy_user"))
+	if(!strcmp((char *)attr->name, "copy_user"))
 	{
-	  if(!strcmp(attr->xmlChildrenNode->content, "1")) launcher->copy_user=1;
-	  if(!strcmp(attr->xmlChildrenNode->content, "2")) launcher->copy_user=2;
+	  if(!strcmp((char *)attr->xmlChildrenNode->content, "1")) launcher->copy_user=1;
+	  if(!strcmp((char *)attr->xmlChildrenNode->content, "2")) launcher->copy_user=2;
 	}
-	if(!strcmp(attr->name, "copy_password"))
+	if(!strcmp((char *)attr->name, "copy_password"))
 	{
-	  if(!strcmp(attr->xmlChildrenNode->content, "1")) launcher->copy_password=1;
-	  if(!strcmp(attr->xmlChildrenNode->content, "2")) launcher->copy_password=2;
+	  if(!strcmp((char *)attr->xmlChildrenNode->content, "1")) launcher->copy_password=1;
+	  if(!strcmp((char *)attr->xmlChildrenNode->content, "2")) launcher->copy_password=2;
 	}
       attr=attr->next;
       }
@@ -459,7 +469,7 @@ passfile_load(gchar* file_name)
     fpm_init_launchers();
   }
 
-  if (list==NULL || strcmp(list->name, "PasswordList"))
+  if (list==NULL || strcmp((char *)list->name, "PasswordList"))
   {
     g_error("Invalid password file.");
     gtk_main_quit();
@@ -488,19 +498,19 @@ passfile_load(gchar* file_name)
     {
       passfile_upd_attr(attr, "title", &data->title);
       passfile_upd_attr(attr, "url", &data->arg);
-      passfile_upd_attr(attr, "arg", &data->arg);
+//      passfile_upd_attr(attr, "arg", &data->arg);
       passfile_upd_attr(attr, "user", &data->user);
       passfile_upd_attr(attr, "notes", &data->notes);
       passfile_upd_attr(attr, "category", &data->category);
       passfile_upd_attr(attr, "launcher", &data->launcher);
-      if(!strcmp(attr->name, "default"))
+      if(!strcmp((char *)attr->name, "default"))
       {
         data->default_list=1;
 	glb_using_defaults=TRUE;
       }
  
-      if(!strcmp(attr->name, "password"))
-        strncpy(data->password, attr->xmlChildrenNode->content, FPM_PASSWORD_LEN*2);
+      if(!strcmp((char *)attr->name, "password"))
+        strncpy(data->password, (char *)attr->xmlChildrenNode->content, FPM_PASSWORD_LEN*2);
 
       attr=attr->next;
     }
@@ -512,7 +522,11 @@ passfile_load(gchar* file_name)
     item=item->next;
     i++;
   }
-  printf("Loaded %d passwords.\n", i);
+  printf("Loaded %d password(s).\n", i);
+
+  dia_str = g_strdup_printf(_("Loaded %d password(s)."), i);
+  fpm_statusbar_push(dia_str);
+
   glb_dirty = FALSE;
 
   return(0);
@@ -657,12 +671,11 @@ passfile_import(gchar *fname)
 	}
 	fclose(fp);
      }
-     dia_str = g_strdup_printf("Imported  %d password(s)", num_added);
-     gnome_ok_dialog( dia_str );
+     dia_str = g_strdup_printf(_("Imported %d password(s)"), num_added);
+     fpm_message(GTK_WINDOW(glb_win_app), dia_str, GTK_MESSAGE_INFO);
      g_free(dia_str);
      printf("Imported %d entries\n", num_added);
      passfile_save(glb_filename);
      fpm_clist_populate_cat_list();
      fpm_check_view(FPM_ALL_CAT_MSG);
-
 }
